"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@collectable/core");
const internals_1 = require("../internals");
function union(other, main) {
    var immutable = core_1.isImmutable(main._owner);
    var outputSet = internals_1.cloneSortedMap(true, main);
    if (Array.isArray(other)) {
        unionArray(main, outputSet, other);
    }
    else if (core_1.isIterable(other)) {
        if (internals_1.isSortedMap(other)) {
            unionSortedMap(main, outputSet, internals_1.iterate(other));
        }
        else {
            unionIterable(main, outputSet, other[Symbol.iterator]());
        }
    }
    else {
        return main;
    }
    if (immutable) {
        return internals_1.refreeze(outputSet);
    }
    main._keyMap = outputSet._keyMap;
    main._sortedValues = outputSet._sortedValues;
    return main;
}
exports.union = union;
function unionArray(inputSet, outputSet, array) {
    var { _keyMap: keyMap, _sortedValues: sortedValues, _select: select } = outputSet;
    for (var i = 0; i < array.length; i++) {
        var value = array[i];
        internals_1.setItem(value[0], value[1], keyMap, sortedValues, select);
    }
}
function unionIterable(inputSet, outputSet, it) {
    var { _keyMap: keyMap, _sortedValues: sortedValues, _select: select } = outputSet;
    var current;
    while (!(current = it.next()).done) {
        var value = current.value;
        internals_1.setItem(value[0], value[1], keyMap, sortedValues, select);
    }
}
function unionSortedMap(inputSet, outputSet, it) {
    var { _keyMap: keyMap, _sortedValues: sortedValues, _select: select } = outputSet;
    var current;
    while (!(current = it.next()).done) {
        var entry = current.value;
        internals_1.setItem(entry.key, entry.value, keyMap, sortedValues, select);
    }
}

//# sourceMappingURL=union.js.map
