"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@collectable/core");
const named_externals_1 = require("./named-externals");
const values_1 = require("./values");
const functions_1 = require("../functions");
const iterate_1 = require("./iterate");
const SORTEDSET_TYPE = {
    type: Symbol('Collectable.SortedMap'),
    indexable: false,
    equals(other, set) {
        return functions_1.isEqual(other, set);
    },
    unwrap(set) {
        return functions_1.unwrap(true, set);
    },
    group(set) {
        return set._group;
    },
    owner(set) {
        return set._owner;
    }
};
class SortedMapImpl {
    constructor(_owner, _group, _keyMap, _sortedValues, _compare, _select) {
        this._owner = _owner;
        this._group = _group;
        this._keyMap = _keyMap;
        this._sortedValues = _sortedValues;
        this._compare = _compare;
        this._select = _select;
    }
    get '@@type'() { return SORTEDSET_TYPE; }
    [Symbol.iterator]() {
        return iterate_1.iteratePairs(this);
    }
}
exports.SortedMapImpl = SortedMapImpl;
function isSortedMap(arg) {
    return arg && arg['@@type'] === SORTEDSET_TYPE;
}
exports.isSortedMap = isSortedMap;
function cloneSortedMap(mutable, set, clear = false) {
    var map;
    var tree;
    if (clear) {
        map = named_externals_1.emptyHashMap(mutable);
        tree = named_externals_1.emptyTree(mutable, set._compare);
    }
    else {
        map = mutable ? named_externals_1.isHashMapThawed(set._keyMap) ? named_externals_1.cloneHashMap(set._keyMap) : named_externals_1.thawHashMap(set._keyMap) : named_externals_1.freezeHashMap(set._keyMap);
        tree = mutable ? named_externals_1.isTreeThawed(set._sortedValues) ? named_externals_1.cloneTree(set._sortedValues) : named_externals_1.thawTree(set._sortedValues) : named_externals_1.freezeTree(set._sortedValues);
    }
    return new SortedMapImpl(core_1.batch.owner(mutable), core_1.nextId(), map, tree, set._compare, set._select);
}
exports.cloneSortedMap = cloneSortedMap;
function cloneAsImmutable(set) {
    return cloneSortedMap(false, set);
}
exports.cloneAsImmutable = cloneAsImmutable;
function cloneAsMutable(set) {
    return cloneSortedMap(true, set);
}
exports.cloneAsMutable = cloneAsMutable;
function refreeze(set) {
    set._owner = 0;
    set._sortedValues._owner = 0;
    set._keyMap._owner = 0;
    return set;
}
exports.refreeze = refreeze;
function createSet(mutable, values, compare, select) {
    var set = emptySet(true, compare, select);
    var map = set._keyMap;
    var tree = set._sortedValues;
    if (Array.isArray(values)) {
        for (var i = 0; i < values.length; i++) {
            var value = values[i];
            values_1.setItem(value[0], value[1], map, tree, select);
        }
    }
    else {
        var it = values[Symbol.iterator]();
        var current;
        while (!(current = it.next()).done) {
            var value = current.value;
            values_1.setItem(value[0], value[1], map, tree, select);
        }
    }
    return refreeze(set);
}
exports.createSet = createSet;
function extractTree(set) {
    return set._sortedValues;
}
exports.extractTree = extractTree;
function extractMap(set) {
    return set._keyMap;
}
exports.extractMap = extractMap;
const DEFAULT_COMPARATOR = (a, b) => a.index - b.index;
const COMPARATOR_CACHE = new WeakMap();
function createComparatorFn(compare) {
    var fn = COMPARATOR_CACHE.get(compare);
    return core_1.isDefined(fn) ? fn : (fn = function (a, b) {
        var n = compare(a, b);
        return n === 0 ? DEFAULT_COMPARATOR(a, b) : n;
    }, COMPARATOR_CACHE.set(compare, fn), fn);
}
function emptySet(mutable = false, compare, select) {
    var comparator = core_1.isDefined(compare) ? createComparatorFn(compare) : DEFAULT_COMPARATOR;
    if (!mutable && comparator === DEFAULT_COMPARATOR) {
        return core_1.isDefined(EMPTY) ? EMPTY : (EMPTY = new SortedMapImpl(0, 0, named_externals_1.emptyHashMap(), named_externals_1.emptyTree(), comparator, void 0));
    }
    var map = named_externals_1.emptyHashMap(mutable);
    var tree = named_externals_1.emptyTree(mutable, comparator);
    return new SortedMapImpl(core_1.batch.owner(mutable), core_1.nextId(), map, tree, comparator, select);
}
exports.emptySet = emptySet;
var EMPTY;

//# sourceMappingURL=SortedMap.js.map
