import { isImmutable, isIterable } from '@collectable/core';
import { isSortedMap, refreeze, cloneSortedMap, unsetItem, iterate } from '../internals';
import { size } from '.';
export function subtract(other, main) {
    var immutable = isImmutable(main._owner);
    var outputSet = cloneSortedMap(true, main);
    if (other && typeof other === 'object') {
        if (Array.isArray(other)) {
            subtractArray(main, outputSet, other);
        }
        else if (isIterable(other)) {
            if (isSortedMap(other)) {
                subtractSortedMap(main, outputSet, iterate(other));
            }
            else {
                subtractIterable(main, outputSet, other[Symbol.iterator]());
            }
        }
        if (size(outputSet) === size(main)) {
            return main;
        }
        if (immutable) {
            main = refreeze(outputSet);
        }
        else {
            main._keyMap = outputSet._keyMap;
            main._sortedValues = outputSet._sortedValues;
        }
    }
    return main;
}
function subtractArray(inputSet, outputSet, omissions) {
    var { _keyMap: keyMap, _sortedValues: sortedValues } = outputSet;
    for (var i = 0; i < omissions.length; i++) {
        var key = Array.isArray(omissions[i]) ? omissions[i][0] : omissions[i];
        unsetItem(key, keyMap, sortedValues);
    }
}
function subtractIterable(inputSet, outputSet, omissions) {
    var { _keyMap: keyMap, _sortedValues: sortedValues } = outputSet;
    var current;
    while (!(current = omissions.next()).done) {
        var key = Array.isArray(current.value) ? current.value[0] : current.value;
        unsetItem(key, keyMap, sortedValues);
    }
}
function subtractSortedMap(inputSet, outputSet, omissions) {
    var { _keyMap: keyMap, _sortedValues: sortedValues } = outputSet;
    var current;
    while (!(current = omissions.next()).done) {
        unsetItem(current.value.key, keyMap, sortedValues);
    }
}

//# sourceMappingURL=subtract.js.map
