import { isImmutable, isIterable } from '@collectable/core';
import { isSortedMap, iterate, refreeze, cloneSortedMap, setItem } from '../internals';
export function union(other, main) {
    var immutable = isImmutable(main._owner);
    var outputSet = cloneSortedMap(true, main);
    if (Array.isArray(other)) {
        unionArray(main, outputSet, other);
    }
    else if (isIterable(other)) {
        if (isSortedMap(other)) {
            unionSortedMap(main, outputSet, iterate(other));
        }
        else {
            unionIterable(main, outputSet, other[Symbol.iterator]());
        }
    }
    else {
        return main;
    }
    if (immutable) {
        return refreeze(outputSet);
    }
    main._keyMap = outputSet._keyMap;
    main._sortedValues = outputSet._sortedValues;
    return main;
}
function unionArray(inputSet, outputSet, array) {
    var { _keyMap: keyMap, _sortedValues: sortedValues, _select: select } = outputSet;
    for (var i = 0; i < array.length; i++) {
        var value = array[i];
        setItem(value[0], value[1], keyMap, sortedValues, select);
    }
}
function unionIterable(inputSet, outputSet, it) {
    var { _keyMap: keyMap, _sortedValues: sortedValues, _select: select } = outputSet;
    var current;
    while (!(current = it.next()).done) {
        var value = current.value;
        setItem(value[0], value[1], keyMap, sortedValues, select);
    }
}
function unionSortedMap(inputSet, outputSet, it) {
    var { _keyMap: keyMap, _sortedValues: sortedValues, _select: select } = outputSet;
    var current;
    while (!(current = it.next()).done) {
        var entry = current.value;
        setItem(entry.key, entry.value, keyMap, sortedValues, select);
    }
}

//# sourceMappingURL=union.js.map
